@@ -17,18 +17,17 @@
 #
 # You should have received a copy of the GNU Lesser Public License
 # along with this program.  If not, see [http://www.gnu.org/licenses/].
-
 """This module contains a object that represents a Telegram Bot."""
 
-import logging
 import functools
+import logging
 
-from telegram import User, Message, Update, UserProfilePhotos, File, \
-    ReplyMarkup, TelegramObject, NullHandler
-from telegram.utils import request
-from telegram.utils.validate import validate_token
+from telegram import (User, Message, Update, Chat, ChatMember, UserProfilePhotos, File,
+                      ReplyMarkup, TelegramObject)
+from telegram.error import InvalidToken
+from telegram.utils.request import Request
 
-logging.getLogger(__name__).addHandler(NullHandler())
+logging.getLogger(__name__).addHandler(logging.NullHandler())
 
 
 class Bot(TelegramObject):
@@ -45,32 +44,45 @@ class Bot(TelegramObject):
         token (str): Bot's unique authentication.
         base_url (Optional[str]): Telegram Bot API service URL.
         base_file_url (Optional[str]): Telegram Bot API file URL.
+        request (Optional[Request]): Pre initialized `Request` class.
 
     """
 
-    def __init__(self,
-                 token,
-                 base_url=None,
-                 base_file_url=None):
-        self.token = validate_token(token)
+    def __init__(self, token, base_url=None, base_file_url=None, request=None):
+        self.token = self._validate_token(token)
 
         if not base_url:
-            self.base_url = 'https://api.telegram.org/bot{0}'.format(
-                self.token)
+            self.base_url = 'https://api.telegram.org/bot{0}'.format(self.token)
         else:
             self.base_url = base_url + self.token
 
         if not base_file_url:
-            self.base_file_url = 'https://api.telegram.org/file/bot{0}'.format(
-                self.token)
+            self.base_file_url = 'https://api.telegram.org/file/bot{0}'.format(self.token)
         else:
             self.base_file_url = base_file_url + self.token
 
         self.bot = None
-
+        self._request = request or Request()
         self.logger = logging.getLogger(__name__)
 
+    @property
+    def request(self):
+        return self._request
+
+    @staticmethod
+    def _validate_token(token):
+        """a very basic validation on token"""
+        if any(x.isspace() for x in token):
+            raise InvalidToken()
+
+        left, sep, _right = token.partition(':')
+        if (not sep) or (not left.isdigit()) or (len(left) < 3):
+            raise InvalidToken()
+
+        return token
+
     def info(func):
+
         @functools.wraps(func)
         def decorator(self, *args, **kwargs):
             if not self.bot:
@@ -119,17 +131,16 @@ class Bot(TelegramObject):
         return decorator
 
     def message(func):
+
         @functools.wraps(func)
         def decorator(self, *args, **kwargs):
             url, data = func(self, *args, **kwargs)
 
             if kwargs.get('reply_to_message_id'):
-                data['reply_to_message_id'] = \
-                    kwargs.get('reply_to_message_id')
+                data['reply_to_message_id'] = kwargs.get('reply_to_message_id')
 
             if kwargs.get('disable_notification'):
-                data['disable_notification'] = \
-                    kwargs.get('disable_notification')
+                data['disable_notification'] = kwargs.get('disable_notification')
 
             if kwargs.get('reply_markup'):
                 reply_markup = kwargs.get('reply_markup')
@@ -138,18 +149,17 @@ class Bot(TelegramObject):
                 else:
                     data['reply_markup'] = reply_markup
 
-            result = request.post(url, data,
-                                  timeout=kwargs.get('timeout'))
+            result = self._request.post(url, data, timeout=kwargs.get('timeout'))
 
             if result is True:
                 return result
 
-            return Message.de_json(result)
+            return Message.de_json(result, self)
 
         return decorator
 
     @log
-    def getMe(self):
+    def getMe(self, **kwargs):
         """A simple method for testing your bot's auth token.
 
         Returns:
@@ -164,20 +174,15 @@ class Bot(TelegramObject):
 
         url = '{0}/getMe'.format(self.base_url)
 
-        result = request.get(url)
+        result = self._request.get(url)
 
-        self.bot = User.de_json(result)
+        self.bot = User.de_json(result, self)
 
         return self.bot
 
     @log
     @message
-    def sendMessage(self,
-                    chat_id,
-                    text,
-                    parse_mode=None,
-                    disable_web_page_preview=None,
-                    **kwargs):
+    def sendMessage(self, chat_id, text, parse_mode=None, disable_web_page_preview=None, **kwargs):
         """Use this method to send text messages.
 
         Args:
@@ -217,8 +222,7 @@ class Bot(TelegramObject):
 
         url = '{0}/sendMessage'.format(self.base_url)
 
-        data = {'chat_id': chat_id,
-                'text': text}
+        data = {'chat_id': chat_id, 'text': text}
 
         if parse_mode:
             data['parse_mode'] = parse_mode
@@ -229,11 +233,7 @@ class Bot(TelegramObject):
 
     @log
     @message
-    def forwardMessage(self,
-                       chat_id,
-                       from_chat_id,
-                       message_id,
-                       **kwargs):
+    def forwardMessage(self, chat_id, from_chat_id, message_id, **kwargs):
         """Use this method to forward messages of any kind.
 
         Args:
@@ -276,11 +276,7 @@ class Bot(TelegramObject):
 
     @log
     @message
-    def sendPhoto(self,
-                  chat_id,
-                  photo,
-                  caption=None,
-                  **kwargs):
+    def sendPhoto(self, chat_id, photo, caption=None, **kwargs):
         """Use this method to send photos.
 
         Args:
@@ -318,8 +314,7 @@ class Bot(TelegramObject):
 
         url = '{0}/sendPhoto'.format(self.base_url)
 
-        data = {'chat_id': chat_id,
-                'photo': photo}
+        data = {'chat_id': chat_id, 'photo': photo}
 
         if caption:
             data['caption'] = caption
@@ -328,13 +323,7 @@ class Bot(TelegramObject):
 
     @log
     @message
-    def sendAudio(self,
-                  chat_id,
-                  audio,
-                  duration=None,
-                  performer=None,
-                  title=None,
-                  **kwargs):
+    def sendAudio(self, chat_id, audio, duration=None, performer=None, title=None, **kwargs):
         """Use this method to send audio files, if you want Telegram clients to
         display them in the music player. Your audio must be in an .mp3 format.
         On success, the sent Message is returned. Bots can currently send audio
@@ -384,8 +373,7 @@ class Bot(TelegramObject):
 
         url = '{0}/sendAudio'.format(self.base_url)
 
-        data = {'chat_id': chat_id,
-                'audio': audio}
+        data = {'chat_id': chat_id, 'audio': audio}
 
         if duration:
             data['duration'] = duration
@@ -398,12 +386,7 @@ class Bot(TelegramObject):
 
     @log
     @message
-    def sendDocument(self,
-                     chat_id,
-                     document,
-                     filename=None,
-                     caption=None,
-                     **kwargs):
+    def sendDocument(self, chat_id, document, filename=None, caption=None, **kwargs):
         """Use this method to send general files.
 
         Args:
@@ -444,8 +427,7 @@ class Bot(TelegramObject):
 
         url = '{0}/sendDocument'.format(self.base_url)
 
-        data = {'chat_id': chat_id,
-                'document': document}
+        data = {'chat_id': chat_id, 'document': document}
 
         if filename:
             data['filename'] = filename
@@ -456,10 +438,7 @@ class Bot(TelegramObject):
 
     @log
     @message
-    def sendSticker(self,
-                    chat_id,
-                    sticker,
-                    **kwargs):
+    def sendSticker(self, chat_id, sticker, **kwargs):
         """Use this method to send .webp stickers.
 
         Args:
@@ -494,19 +473,13 @@ class Bot(TelegramObject):
 
         url = '{0}/sendSticker'.format(self.base_url)
 
-        data = {'chat_id': chat_id,
-                'sticker': sticker}
+        data = {'chat_id': chat_id, 'sticker': sticker}
 
         return url, data
 
     @log
     @message
-    def sendVideo(self,
-                  chat_id,
-                  video,
-                  duration=None,
-                  caption=None,
-                  **kwargs):
+    def sendVideo(self, chat_id, video, duration=None, caption=None, **kwargs):
         """Use this method to send video files, Telegram clients support mp4
         videos (other formats may be sent as telegram.Document).
 
@@ -547,8 +520,7 @@ class Bot(TelegramObject):
 
         url = '{0}/sendVideo'.format(self.base_url)
 
-        data = {'chat_id': chat_id,
-                'video': video}
+        data = {'chat_id': chat_id, 'video': video}
 
         if duration:
             data['duration'] = duration
@@ -559,11 +531,7 @@ class Bot(TelegramObject):
 
     @log
     @message
-    def sendVoice(self,
-                  chat_id,
-                  voice,
-                  duration=None,
-                  **kwargs):
+    def sendVoice(self, chat_id, voice, duration=None, **kwargs):
         """Use this method to send audio files, if you want Telegram clients to
         display the file as a playable voice message. For this to work, your
         audio must be in an .ogg file encoded with OPUS (other formats may be
@@ -605,8 +573,7 @@ class Bot(TelegramObject):
 
         url = '{0}/sendVoice'.format(self.base_url)
 
-        data = {'chat_id': chat_id,
-                'voice': voice}
+        data = {'chat_id': chat_id, 'voice': voice}
 
         if duration:
             data['duration'] = duration
@@ -615,11 +582,7 @@ class Bot(TelegramObject):
 
     @log
     @message
-    def sendLocation(self,
-                     chat_id,
-                     latitude,
-                     longitude,
-                     **kwargs):
+    def sendLocation(self, chat_id, latitude, longitude, **kwargs):
         """Use this method to send point on the map.
 
         Args:
@@ -654,21 +617,13 @@ class Bot(TelegramObject):
 
         url = '{0}/sendLocation'.format(self.base_url)
 
-        data = {'chat_id': chat_id,
-                'latitude': latitude,
-                'longitude': longitude}
+        data = {'chat_id': chat_id, 'latitude': latitude, 'longitude': longitude}
 
         return url, data
 
     @log
     @message
-    def sendVenue(self,
-                  chat_id,
-                  latitude,
-                  longitude,
-                  title,
-                  address,
-                  foursquare_id=None,
+    def sendVenue(self, chat_id, latitude, longitude, title, address, foursquare_id=None,
                   **kwargs):
         """
         Use this method to send information about a venue.
@@ -725,12 +680,7 @@ class Bot(TelegramObject):
 
     @log
     @message
-    def sendContact(self,
-                    chat_id,
-                    phone_number,
-                    first_name,
-                    last_name=None,
-                    **kwargs):
+    def sendContact(self, chat_id, phone_number, first_name, last_name=None, **kwargs):
         """
         Use this method to send phone contacts.
 
@@ -769,9 +719,7 @@ class Bot(TelegramObject):
 
         url = '{0}/sendContact'.format(self.base_url)
 
-        data = {'chat_id': chat_id,
-                'phone_number': phone_number,
-                'first_name': first_name}
+        data = {'chat_id': chat_id, 'phone_number': phone_number, 'first_name': first_name}
 
         if last_name:
             data['last_name'] = last_name
@@ -780,10 +728,7 @@ class Bot(TelegramObject):
 
     @log
     @message
-    def sendChatAction(self,
-                       chat_id,
-                       action,
-                       **kwargs):
+    def sendChatAction(self, chat_id, action, **kwargs):
         """Use this method when you need to tell the user that something is
         happening on the bot's side. The status is set for 5 seconds or less
         (when a message arrives from your bot, Telegram clients clear its
@@ -805,8 +750,7 @@ class Bot(TelegramObject):
 
         url = '{0}/sendChatAction'.format(self.base_url)
 
-        data = {'chat_id': chat_id,
-                'action': action}
+        data = {'chat_id': chat_id, 'action': action}
 
         return url, data
 
@@ -861,8 +805,7 @@ class Bot(TelegramObject):
 
         results = [res.to_dict() for res in results]
 
-        data = {'inline_query_id': inline_query_id,
-                'results': results}
+        data = {'inline_query_id': inline_query_id, 'results': results}
 
         if cache_time or cache_time == 0:
             data['cache_time'] = cache_time
@@ -875,17 +818,12 @@ class Bot(TelegramObject):
         if switch_pm_parameter:
             data['switch_pm_parameter'] = switch_pm_parameter
 
-        result = request.post(url, data,
-                              timeout=kwargs.get('timeout'))
+        result = self._request.post(url, data, timeout=kwargs.get('timeout'))
 
         return result
 
     @log
-    def getUserProfilePhotos(self,
-                             user_id,
-                             offset=None,
-                             limit=100,
-                             **kwargs):
+    def getUserProfilePhotos(self, user_id, offset=None, limit=100, **kwargs):
         """Use this method to get a list of profile pictures for a user.
 
         Args:
@@ -920,15 +858,12 @@ class Bot(TelegramObject):
         if limit:
             data['limit'] = limit
 
-        result = request.post(url, data,
-                              timeout=kwargs.get('timeout'))
+        result = self._request.post(url, data, timeout=kwargs.get('timeout'))
 
-        return UserProfilePhotos.de_json(result)
+        return UserProfilePhotos.de_json(result, self)
 
     @log
-    def getFile(self,
-                file_id,
-                **kwargs):
+    def getFile(self, file_id, **kwargs):
         """Use this method to get basic info about a file and prepare it for
         downloading. For the moment, bots can download files of up to 20MB in
         size.
@@ -954,20 +889,15 @@ class Bot(TelegramObject):
 
         data = {'file_id': file_id}
 
-        result = request.post(url, data,
-                              timeout=kwargs.get('timeout'))
+        result = self._request.post(url, data, timeout=kwargs.get('timeout'))
 
         if result.get('file_path'):
-            result['file_path'] = '%s/%s' % (self.base_file_url,
-                                             result['file_path'])
+            result['file_path'] = '%s/%s' % (self.base_file_url, result['file_path'])
 
-        return File.de_json(result)
+        return File.de_json(result, self)
 
     @log
-    def kickChatMember(self,
-                       chat_id,
-                       user_id,
-                       **kwargs):
+    def kickChatMember(self, chat_id, user_id, **kwargs):
         """Use this method to kick a user from a group or a supergroup. In the
         case of supergroups, the user will not be able to return to the group
         on their own using invite links, etc., unless unbanned first. The bot
@@ -994,19 +924,14 @@ class Bot(TelegramObject):
 
         url = '{0}/kickChatMember'.format(self.base_url)
 
-        data = {'chat_id': chat_id,
-                'user_id': user_id}
+        data = {'chat_id': chat_id, 'user_id': user_id}
 
-        result = request.post(url, data,
-                              timeout=kwargs.get('timeout'))
+        result = self._request.post(url, data, timeout=kwargs.get('timeout'))
 
         return result
 
     @log
-    def unbanChatMember(self,
-                        chat_id,
-                        user_id,
-                        **kwargs):
+    def unbanChatMember(self, chat_id, user_id, **kwargs):
         """Use this method to unban a previously kicked user in a supergroup.
         The user will not return to the group automatically, but will be able
         to join via link, etc. The bot must be an administrator in the group
@@ -1033,20 +958,14 @@ class Bot(TelegramObject):
 
         url = '{0}/unbanChatMember'.format(self.base_url)
 
-        data = {'chat_id': chat_id,
-                'user_id': user_id}
+        data = {'chat_id': chat_id, 'user_id': user_id}
 
-        result = request.post(url, data,
-                              timeout=kwargs.get('timeout'))
+        result = self._request.post(url, data, timeout=kwargs.get('timeout'))
 
         return result
 
     @log
-    def answerCallbackQuery(self,
-                            callback_query_id,
-                            text=None,
-                            show_alert=False,
-                            **kwargs):
+    def answerCallbackQuery(self, callback_query_id, text=None, show_alert=False, **kwargs):
         """Use this method to send answers to callback queries sent from
         inline keyboards. The answer will be displayed to the user as a
         notification at the top of the chat screen or as an alert.
@@ -1085,12 +1004,12 @@ class Bot(TelegramObject):
         if show_alert:
             data['show_alert'] = show_alert
 
-        result = request.post(url, data,
-                              timeout=kwargs.get('timeout'))
+        result = self._request.post(url, data, timeout=kwargs.get('timeout'))
 
         return result
 
     @log
+    @message
     def editMessageText(self,
                         text,
                         chat_id=None,
@@ -1098,7 +1017,6 @@ class Bot(TelegramObject):
                         inline_message_id=None,
                         parse_mode=None,
                         disable_web_page_preview=None,
-                        reply_markup=None,
                         **kwargs):
         """Use this method to edit text messages sent by the bot or via the bot
         (for inline bots).
@@ -1125,6 +1043,10 @@ class Bot(TelegramObject):
             A JSON-serialized object for an inline keyboard.
 
         Keyword Args:
+            reply_markup (Optional[:class:`telegram.ReplyMarkup`]): Additional
+                interface options. A JSON-serialized object for an inline
+                keyboard, custom reply keyboard, instructions to hide reply
+                keyboard or to force a reply from the user.
             timeout (Optional[float]): If this value is specified, use it as
                 the definitive timeout (in seconds) for urlopen() operations.
 
@@ -1152,16 +1074,8 @@ class Bot(TelegramObject):
             data['parse_mode'] = parse_mode
         if disable_web_page_preview:
             data['disable_web_page_preview'] = disable_web_page_preview
-        if reply_markup:
-            if isinstance(reply_markup, ReplyMarkup):
-                data['reply_markup'] = reply_markup.to_json()
-            else:
-                data['reply_markup'] = reply_markup
 
-        result = request.post(url, data,
-                              timeout=kwargs.get('timeout'))
-
-        return Message.de_json(result)
+        return url, data
 
     @log
     @message
@@ -1266,35 +1180,27 @@ class Bot(TelegramObject):
         return url, data
 
     @log
-    def getUpdates(self,
-                   offset=None,
-                   limit=100,
-                   timeout=0,
-                   network_delay=.2):
+    def getUpdates(self, offset=None, limit=100, timeout=0, network_delay=5., **kwargs):
         """Use this method to receive incoming updates using long polling.
 
         Args:
-          offset:
-            Identifier of the first update to be returned. Must be greater by
-            one than the highest among the identifiers of previously received
-            updates. By default, updates starting with the earliest unconfirmed
-            update are returned. An update is considered confirmed as soon as
-            getUpdates is called with an offset higher than its update_id.
-          limit:
-            Limits the number of updates to be retrieved. Values between 1-100
-            are accepted. Defaults to 100.
-          timeout:
-            Timeout in seconds for long polling. Defaults to 0, i.e. usual
-            short polling.
-          network_delay:
-            Additional timeout in seconds to allow the response from Telegram
-            to take some time when using long polling. Defaults to 2, which
-            should be enough for most connections. Increase it if it takes very
-            long for data to be transmitted from and to the Telegram servers.
+          offset (Optional[int]):
+            Identifier of the first update to be returned. Must be greater by  one than the highest
+            among the identifiers of previously received updates. By default, updates starting with
+            the earliest unconfirmed update are returned. An update is considered confirmed as soon
+            as getUpdates is called with an offset higher than its update_id.
+          limit (Optional[int]):
+            Limits the number of updates to be retrieved. Values between 1-100 are accepted.
+            Defaults to 100.
+          timeout (Optional[int]):
+            Timeout in seconds for long polling. Defaults to 0, i.e. usual short polling.
+          network_delay (Optional[float]):
+            Additional timeout in seconds to allow the response from Telegram servers. This should
+            cover network latency around the globe, SSL handshake and slowness of the Telegram
+            servers (which unfortunately happens a lot recently - 2016-05-28). Defaults to 5.
 
         Returns:
-            list[:class:`telegram.Message`]: A list of :class:`telegram.Update`
-            objects are returned.
+            list[:class:`telegram.Update`]
 
         Raises:
             :class:`telegram.TelegramError`
@@ -1312,21 +1218,17 @@ class Bot(TelegramObject):
 
         urlopen_timeout = timeout + network_delay
 
-        result = request.post(url, data, timeout=urlopen_timeout)
+        result = self._request.post(url, data, timeout=urlopen_timeout)
 
         if result:
-            self.logger.debug(
-                'Getting updates: %s', [u['update_id'] for u in result])
+            self.logger.debug('Getting updates: %s', [u['update_id'] for u in result])
         else:
             self.logger.debug('No new updates found.')
 
-        return [Update.de_json(x) for x in result]
+        return [Update.de_json(u, self) for u in result]
 
     @log
-    def setWebhook(self,
-                   webhook_url=None,
-                   certificate=None,
-                   **kwargs):
+    def setWebhook(self, webhook_url=None, certificate=None, **kwargs):
         """Use this method to specify a url and receive incoming updates via an
         outgoing webhook. Whenever there is an update for the bot, we will send
         an HTTPS POST request to the specified url, containing a
@@ -1359,21 +1261,178 @@ class Bot(TelegramObject):
         if certificate:
             data['certificate'] = certificate
 
-        result = request.post(url, data,
-                              timeout=kwargs.get('timeout'))
+        result = self._request.post(url, data, timeout=kwargs.get('timeout'))
 
         return result
 
+    @log
+    def leaveChat(self, chat_id, **kwargs):
+        """Use this method for your bot to leave a group, supergroup or
+        channel.
+
+        Args:
+          chat_id:
+            Unique identifier for the target chat or username of the target
+            channel (in the format @channelusername).
+
+        Keyword Args:
+            timeout (Optional[float]): If this value is specified, use it as
+                the definitive timeout (in seconds) for urlopen() operations.
+
+        Returns:
+            bool: On success, `True` is returned.
+
+        Raises:
+            :class:`telegram.TelegramError`
+
+        """
+
+        url = '{0}/leaveChat'.format(self.base_url)
+
+        data = {'chat_id': chat_id}
+
+        result = self._request.post(url, data, timeout=kwargs.get('timeout'))
+
+        return result
+
+    @log
+    def getChat(self, chat_id, **kwargs):
+        """Use this method to get up to date information about the chat
+        (current name of the user for one-on-one conversations, current
+        username of a user, group or channel, etc.).
+
+        Args:
+          chat_id:
+            Unique identifier for the target chat or username of the target
+            channel (in the format @channelusername).
+
+        Keyword Args:
+            timeout (Optional[float]): If this value is specified, use it as
+                the definitive timeout (in seconds) for urlopen() operations.
+
+        Returns:
+            :class:`telegram.Chat`: On success, :class:`telegram.Chat` is
+            returned.
+
+        Raises:
+            :class:`telegram.TelegramError`
+
+        """
+
+        url = '{0}/getChat'.format(self.base_url)
+
+        data = {'chat_id': chat_id}
+
+        result = self._request.post(url, data, timeout=kwargs.get('timeout'))
+
+        return Chat.de_json(result, self)
+
+    @log
+    def getChatAdministrators(self, chat_id, **kwargs):
+        """Use this method to get a list of administrators in a chat. On
+        success, returns an Array of ChatMember objects that contains
+        information about all chat administrators except other bots. If the
+        chat is a group or a supergroup and no administrators were appointed,
+        only the creator will be returned.
+
+        Args:
+          chat_id:
+            Unique identifier for the target chat or username of the target
+            channel (in the format @channelusername).
+
+
+        Keyword Args:
+            timeout (Optional[float]): If this value is specified, use it as
+                the definitive timeout (in seconds) for urlopen() operations.
+
+        Returns:
+            list[:class:`telegram.ChatMember`]: On success, a list of
+            :class:`telegram.ChatMember` objects are returned.
+
+        Raises:
+            :class:`telegram.TelegramError`
+
+        """
+
+        url = '{0}/getChatAdministrators'.format(self.base_url)
+
+        data = {'chat_id': chat_id}
+
+        result = self._request.post(url, data, timeout=kwargs.get('timeout'))
+
+        return [ChatMember.de_json(x, self) for x in result]
+
+    @log
+    def getChatMembersCount(self, chat_id, **kwargs):
+        """Use this method to get the number of members in a chat.
+
+        Args:
+          chat_id:
+            Unique identifier for the target chat or username of the target
+            channel (in the format @channelusername).
+
+
+        Keyword Args:
+            timeout (Optional[float]): If this value is specified, use it as
+                the definitive timeout (in seconds) for urlopen() operations.
+
+        Returns:
+            int: On success, an `int` is returned.
+
+        Raises:
+            :class:`telegram.TelegramError`
+
+        """
+
+        url = '{0}/getChatMembersCount'.format(self.base_url)
+
+        data = {'chat_id': chat_id}
+
+        result = self._request.post(url, data, timeout=kwargs.get('timeout'))
+
+        return result
+
+    @log
+    def getChatMember(self, chat_id, user_id, **kwargs):
+        """Use this method to get information about a member of a chat.
+
+        Args:
+          chat_id:
+            Unique identifier for the target chat or username of the target
+            channel (in the format @channelusername).
+          user_id:
+            Unique identifier of the target user.
+
+
+        Keyword Args:
+            timeout (Optional[float]): If this value is specified, use it as
+                the definitive timeout (in seconds) for urlopen() operations.
+
+        Returns:
+            :class:`telegram.ChatMember`: On success,
+            :class:`telegram.ChatMember` is returned.
+
+        Raises:
+            :class:`telegram.TelegramError`
+
+        """
+
+        url = '{0}/getChatMember'.format(self.base_url)
+
+        data = {'chat_id': chat_id, 'user_id': user_id}
+
+        result = self._request.post(url, data, timeout=kwargs.get('timeout'))
+
+        return ChatMember.de_json(result, self)
+
     @staticmethod
-    def de_json(data):
-        data = super(Bot, Bot).de_json(data)
+    def de_json(data, bot):
+        data = super(Bot, Bot).de_json(data, bot)
 
         return Bot(**data)
 
     def to_dict(self):
-        data = {'id': self.id,
-                'username': self.username,
-                'first_name': self.username}
+        data = {'id': self.id, 'username': self.username, 'first_name': self.username}
 
         if self.last_name:
             data['last_name'] = self.last_name
@@ -1381,6 +1440,36 @@ class Bot(TelegramObject):
         return data
 
     def __reduce__(self):
-        return (self.__class__, (self.token,
-                                 self.base_url.replace(self.token, ''),
+        return (self.__class__, (self.token, self.base_url.replace(self.token, ''),
                                  self.base_file_url.replace(self.token, '')))
+
+    # snake_case (PEP8) aliases
+    get_me = getMe
+    send_message = sendMessage
+    forward_message = forwardMessage
+    send_photo = sendPhoto
+    send_audio = sendAudio
+    send_document = sendDocument
+    send_sticker = sendSticker
+    send_video = sendVideo
+    send_voice = sendVoice
+    send_location = sendLocation
+    send_venue = sendVenue
+    send_contact = sendContact
+    send_chat_action = sendChatAction
+    answer_inline_query = answerInlineQuery
+    get_user_profile_photos = getUserProfilePhotos
+    get_file = getFile
+    kick_chat_member = kickChatMember
+    unban_chat_member = unbanChatMember
+    answer_callback_query = answerCallbackQuery
+    edit_message_text = editMessageText
+    edit_message_caption = editMessageCaption
+    edit_message_reply_markup = editMessageReplyMarkup
+    get_updates = getUpdates
+    set_webhook = setWebhook
+    leave_chat = leaveChat
+    get_chat = getChat
+    get_chat_administrators = getChatAdministrators
+    get_chat_member = getChatMember
+    get_chat_members_count = getChatMembersCount
