@@ -41,9 +41,10 @@ class User(TelegramObject):
         type (Optional[str]):
         last_name (Optional[str]):
         username (Optional[str]):
+        bot (Optional[Bot]): The Bot to use for instance methods
     """
 
-    def __init__(self, id, first_name, **kwargs):
+    def __init__(self, id, first_name, bot=None, **kwargs):
         # Required
         self.id = int(id)
         self.first_name = first_name
@@ -52,6 +53,8 @@ class User(TelegramObject):
         self.last_name = kwargs.get('last_name', '')
         self.username = kwargs.get('username', '')
 
+        self.bot = bot
+
     @property
     def name(self):
         """str: """
@@ -62,10 +65,11 @@ class User(TelegramObject):
         return self.first_name
 
     @staticmethod
-    def de_json(data):
+    def de_json(data, bot):
         """
         Args:
-            data (str):
+            data (dict):
+            bot (telegram.Bot):
 
         Returns:
             telegram.User:
@@ -73,4 +77,10 @@ class User(TelegramObject):
         if not data:
             return None
 
-        return User(**data)
+        return User(bot=bot, **data)
+
+    def get_profile_photos(self, *args, **kwargs):
+        """
+        Shortcut for ``bot.getUserProfilePhotos(update.message.from_user.id, *args, **kwargs)``
+        """
+        return self.bot.getUserProfilePhotos(self.id, *args, **kwargs)
